/*
	file automatically generated by make_test_files.pl
	Fri Jun 14 15:21:41 2002
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for STRINGPARSER
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_STRINGPARSER
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_stringparser_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */
int unur_ssr_set_pedantic( struct unur_par *par, int pedantic );

static int COMPARE_SAMPLE_SIZE = 10000;
static int VIOLATE_SAMPLE_SIZE = 20;





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_stringparser_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_stringparser_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(5678)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("stringparser: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 47, (unur_str2gen( NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 47, UNUR_ERR_NULL );
}

/* invalid distribution block */
{

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 53, (unur_str2gen( "xxxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 53, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 56, (unur_str2gen( "method = xxxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 56, UNUR_ERR_STR_SYNTAX );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 59, (unur_str2gen( "normal; xxxx=1" )) );
n_tests_failed += check_errorcode( TESTLOG, 59, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 62, (unur_str2gen( "normal; pv=(1,2,3)" )) );
n_tests_failed += check_errorcode( TESTLOG, 62, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 65, (unur_str2gen( "normal,(1,2)" )) );
n_tests_failed += check_errorcode( TESTLOG, 65, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 68, (unur_str2gen( "beta" )) );
n_tests_failed += check_errorcode( TESTLOG, 68, UNUR_ERR_STR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 71, (unur_str2gen( "beta()" )) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_STR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 74, (unur_str2gen( "beta(1)" )) );
n_tests_failed += check_errorcode( TESTLOG, 74, UNUR_ERR_STR_INVALID );

unur_errno = 0;
unur_str2gen( "beta(1,2,3)" );
n_tests_failed += check_errorcode( TESTLOG, 77, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
unur_str2gen( "beta(1,2,3,4,5)" );
n_tests_failed += check_errorcode( TESTLOG, 80, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 83, (unur_str2gen( "gamma(-0.5)" )) );
n_tests_failed += check_errorcode( TESTLOG, 83, UNUR_ERR_STR_INVALID );
}

/* invalid other block */
{

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 89, (unur_str2gen( "normal & xxxx = arou" )) );
n_tests_failed += check_errorcode( TESTLOG, 89, UNUR_ERR_STR_UNKNOWN );
}

/* invalid method block */
{
	UNUR_GEN *gen = NULL; 

unur_errno = 0;
gen = unur_str2gen( "normal & method = arou; c = 0" );
n_tests_failed += check_errorcode( TESTLOG, 96, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 100, (gen = unur_str2gen( "normal & method = xxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 100, UNUR_ERR_NULL );
}

/* invalid urng block */
{
	UNUR_GEN *gen = NULL; 

unur_errno = 0;
gen = unur_str2gen( "normal & urng = xxx" );
n_tests_failed += check_errorcode( TESTLOG, 107, UNUR_ERR_STR );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "normal & urng = mt19937(123); xxx = 1" );
n_tests_failed += check_errorcode( TESTLOG, 111, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 116, 0x0u );
}

/* invalid data */
{

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 121, (unur_str2gen( "gamma(0.5) & method = tdr" )) );
n_tests_failed += check_errorcode( TESTLOG, 121, UNUR_ERR_GEN_CONDITION );
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare function parser - cont */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_normal(NULL,0); 
par = unur_srou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 142, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = srou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 148, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_ssr_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 155, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = ssr" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 161, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_utdr_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 168, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = utdr" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 174, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_tdr_new(distr);
unur_tdr_set_cpoints(par,8,NULL);
unur_tdr_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 183, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = tdr; cpoints = 8; max_sqhratio = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 189, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_arou_new(distr);
unur_arou_set_cpoints(par,8,NULL);
unur_arou_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 198, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = arou; cpoints = 8; max_sqhratio = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 204, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_exponential(NULL,0); 
par = unur_ninv_new(distr);
unur_ninv_set_usenewton(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 212, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; cdf = \"1-exp(-x)\"; domain = (0,inf)& \
	             method = ninv; usenewton" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 218, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 222, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare function parser - discr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {0.4};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_geometric(fpar,1); 
par = unur_dsrou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 235, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dsrou" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 241, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dau_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 248, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dau" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 254, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dgt_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 261, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dgt" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 267, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dari_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 274, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dari" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 280, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 284, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare auto - cont */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_normal(NULL,0); 
par = unur_auto_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 296, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal()" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 301, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal() & method = auto" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 306, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 310, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare auto - discr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {0.4};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_geometric(fpar,1); 
par = unur_auto_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 321, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "geometric(0.4)");
n_tests_failed += compare_int_sequence_gen( TESTLOG, 326, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "geometric(0.4) & method = auto");
n_tests_failed += compare_int_sequence_gen( TESTLOG, 331, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 335, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare empk */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double data[] = {1.,2.,3.,4.,5.,6.,7.,8.,9.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_cemp_new();
unur_distr_cemp_set_data(distr,data,9);
par = unur_auto_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 347, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cemp; data = (1.,2.,3.,4.,5.,6.,7.,8.,9.)" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 352, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cemp; data = (1.,2.,3.,4.,5.,6.,7.,8.,9.) & \
  method = auto" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 358, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 362, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare arou */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[10];
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_normal(NULL,0); 
par = unur_arou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 375, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 380, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "normal() & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 384, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "normal(0.,1.) & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 388, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "normal(0,1) & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 392, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "distr = normal(0,1) & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 396, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
fpar[0] = 1.;
fpar[1] = 2.;
distr = unur_distr_normal(fpar,2); 
par = unur_arou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 404, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal(1,2) & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 409, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
fpar[0] = 1.;
fpar[1] = 2.;
distr = unur_distr_normal(fpar,2);
unur_distr_cont_set_domain(distr,0,2.); 
par = unur_arou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 418, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal(1,2); domain = 0, 2 & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 423, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "normal(1,2); domain = (0,2) & method = arou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 427, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_arou_new(distr);
fpar[0] = -2;
fpar[1] = -1.5;
fpar[2] = -1;
fpar[3] = -0.5;
fpar[4] = 0;
fpar[5] = 0.5;
fpar[6] = 1;
fpar[7] = 1.5;
fpar[8] = 2;
unur_arou_set_cpoints(par,9,fpar);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 443, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal() & \
  method = arou; cpoints = (-2,-1.5,-1,-0.5,0,0.5,1,1.5,2)" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 449, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_arou_new(distr);
unur_arou_set_center(par,0);
unur_arou_set_cpoints(par,20,NULL);
unur_arou_set_guidefactor(par,1.);
unur_arou_set_max_segments(par,40);
unur_arou_set_max_sqhratio(par,0.5);
unur_arou_set_pedantic(par,FALSE);
unur_arou_set_usecenter(par,TRUE);
unur_arou_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 463, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal() & \
  method = arou; \
  center = 0; cpoints = 20; guidefactor = 1.; max_segments = 40; \
  max_sqhratio = 0.5; pedantic = off; usecenter = true; verify = on" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 471, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 475, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare cstd */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {3., 4.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
par = unur_cstd_new(distr);
unur_cstd_set_variant(par,2);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 489, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,4.) & method = cstd; variant = 2" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 494, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 499, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare dari */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {20, 0.8};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_binomial(fpar,2);
par = unur_dari_new(distr);
unur_dari_set_cpfactor(par,0.7);
unur_dari_set_squeeze(par,TRUE);
unur_dari_set_tablesize(par,30);
unur_dari_set_verify(par,FALSE);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 516, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "binomial(20,0.8) & \
  method = dari; cpfactor = 0.7; squeeze; tablesize = 30; verify = off" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 522, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 527, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare dau */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double pv[] = {.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.};
   double fpar[] = {0.8};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_discr_new();
unur_distr_discr_set_pv(distr,pv,10);
par = unur_dau_new(distr);
unur_dau_set_urnfactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 543, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pv = (.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.) & \
  method = dau; urnfactor = 3." );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 549, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_geometric(fpar,1);
par = unur_dau_new(distr);
unur_dau_set_urnfactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 557, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "geometric(0.8) & method = dau; urnfactor = 3." );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 562, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 566, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare dgt */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double pv[] = {.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.};
   double fpar[] = {0.8};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_discr_new();
unur_distr_discr_set_pv(distr,pv,10);
par = unur_dgt_new(distr);
unur_dgt_set_guidefactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 582, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pv = (.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.) & \
  method = dgt; guidefactor = 3." );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 588, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_discr_new();
unur_distr_discr_set_pv(distr,pv,10);
par = unur_dgt_new(distr);
unur_dgt_set_variant(par,1);
unur_dgt_set_guidefactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 598, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pv = (.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.) & \
  method = dgt; guidefactor = 3.; variant = 1" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 604, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_free(gen); gen = NULL;
distr = unur_distr_discr_new();
unur_distr_discr_set_pv(distr,pv,10);
par = unur_dgt_new(distr);
unur_dgt_set_variant(par,2);
unur_dgt_set_guidefactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 615, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pv = (.1, .2, .3, .4, .5, .6, .7, .8, .9, 1.) & \
  method = dgt; guidefactor = 3.; variant = 2" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 621, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_geometric(fpar,1);
par = unur_dgt_new(distr);
unur_dgt_set_guidefactor(par,3.);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 629, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "geometric(0.8) & method = dgt; guidefactor = 3." );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 634, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 638, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare dstd */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_poisson(fpar,1);
par = unur_dstd_new(distr);
unur_dstd_set_variant(par,2);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 652, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "poisson(3.) & method = dstd; variant = 2" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 657, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 662, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare empk */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double data[] = {1.,2.,3.,4.,5.,6.,7.,8.,9.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_cemp_new();
unur_distr_cemp_set_data(distr,data,9);
par = unur_empk_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 676, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cemp; data = (1.,2.,3.,4.,5.,6.,7.,8.,9.) & \
  method = empk" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 682, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_cemp_new();
unur_distr_cemp_set_data(distr,data,9);
par = unur_empk_new(distr);
unur_empk_set_beta(par,1.2);
unur_empk_set_kernel(par,UNUR_DISTR_UNIFORM);
unur_empk_set_positive(par,TRUE);
unur_empk_set_smoothing(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 693, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cemp; data = (1.,2.,3.,4.,5.,6.,7.,8.,9.) & \
  method = empk; beta = 1.2; kernel = 200; positive; smoothing = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 699, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_cemp_new();
unur_distr_cemp_set_data(distr,data,9);
par = unur_empk_new(distr);
unur_empk_set_varcor(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 707, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cemp; data = (1.,2.,3.,4.,5.,6.,7.,8.,9.) & \
  method = empk; varcor" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 713, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 717, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare ninv */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {3.,4.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_ninv_new(distr);
unur_ninv_set_max_iter(par,10);
unur_ninv_set_start(par,4.,10.);
unur_ninv_set_table(par,100);
unur_ninv_set_usenewton(par);
unur_ninv_set_x_resolution(par,1.E-6);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 738, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,4.); domain = 2,infinity & \
  method = ninv; max_iter = 10; start = 4., 10.; table = 100; \
  usenewton; x_resolution = 1.E-6; debug = 0x1" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 745, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_ninv_new(distr);
unur_ninv_set_useregula(par);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 754, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,4.); domain = (2,inf) & method = ninv; useregula; debug = 0x1" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 759, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 764, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare srou */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_beta(fpar,2);
unur_distr_cont_set_pdfarea(distr,2);
par = unur_srou_new(distr);
unur_srou_set_cdfatmode(par,0.5);
unur_srou_set_pdfatmode(par,1.9);
unur_srou_set_usemirror(par,TRUE);
unur_srou_set_usesqueeze(par,FALSE);
unur_srou_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 783, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.); pdfarea = 2. & \
  method = srou; cdfatmode = 0.5; usemirror; usesqueeze = off; pdfatmode = 1.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 789, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_beta(fpar,2);
unur_distr_cont_set_pdfarea(distr,2);
par = unur_srou_new(distr);
unur_srou_set_r(par,3.);
unur_srou_set_cdfatmode(par,0.5);
unur_srou_set_pdfatmode(par,1.9);
unur_srou_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 800, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.); pdfarea = 2. & \
  method = srou; r=3; cdfatmode = 0.5; pdfatmode = 1.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 806, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 810, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare ssr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_beta(fpar,2);
unur_distr_cont_set_pdfarea(distr,2);
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_pdfatmode(par,1.9);
unur_ssr_set_usesqueeze(par,FALSE);
unur_ssr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 828, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.); pdfarea = 2. & \
  method = ssr; cdfatmode = 0.5; usesqueeze = off; pdfatmode = 1.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 834, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 838, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare table */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   double slopes[] = {0.5,-1,0.5,1};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_beta(fpar,2);
par = unur_tabl_new(distr);
unur_tabl_set_areafraction(par,0.25);
unur_tabl_set_boundary(par,0.1,0.9);
unur_tabl_set_guidefactor(par,5);
unur_tabl_set_max_intervals(par,50);
unur_tabl_set_max_sqhratio(par,0.8);
unur_tabl_set_nstp(par,30);
unur_tabl_set_variant_setup(par,1);
unur_tabl_set_variant_splitmode(par,3);
unur_tabl_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 861, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.) & \
   method = tabl; areafraction = 0.25; boundary = (0.1,0.9); \
   guidefactor = 5; max_intervals = 50; max_sqhratio = 0.8; \
   nstp = 30; variant_setup = 1; variant_splitmode = 3; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 869, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_beta(fpar,2);
par = unur_tabl_new(distr);
unur_tabl_set_slopes(par,slopes,2);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 877, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.) & \
   method = tabl; slopes = (0.5,-1,0.5,1),2 " );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 883, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 887, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare tdr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_center(par,2.1);
unur_tdr_set_cpoints(par,20,NULL);
unur_tdr_set_guidefactor(par,2.);
unur_tdr_set_max_intervals(par,32);
unur_tdr_set_max_sqhratio(par,0.9);
unur_tdr_set_usecenter(par,TRUE);
unur_tdr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 907, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; center = 2.1; usecenter; cpoints = 20; guidefactor = 2.; \
  max_intervals = 32; max_sqhratio = 0.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 914, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_cpoints(par,9,NULL);
unur_tdr_set_pedantic(par,TRUE);
unur_tdr_set_usemode(par,FALSE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 924, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; cpoints = 9; pedantic; usemode = false" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 930, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 938, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 944, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_c(par,-0.5);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 952, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; c = -0.5" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 958, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_gw(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 966, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_gw" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 972, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ia(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 980, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ia" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 986, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_gamma(fpar,2);
par = unur_tdr_new(distr);
unur_tdr_set_variant_ps(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 994, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = tdr; variant_ps" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 1000, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 1004, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}

/* compare unif */
{
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
par = unur_unif_new(NULL);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 1014, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "uniform & method = unif" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 1018, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 1022, 0x0u );
unur_free(gen);
}

/* compare utdr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
par = unur_utdr_new(distr);
unur_utdr_set_cpfactor(par,0.660);
unur_utdr_set_deltafactor(par,1.e-4);
unur_utdr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 1038, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,3.) & \
  method = utdr; cpfactor = 0.66; deltafactor = 1.e-4; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 1044, urng, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_errorcode( TESTLOG, 1048, 0x0u );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/






/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_stringparser_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_stringparser_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_STRINGPARSER */
/*---------------------------------------------------------------------------*/
